#!/usr/bin/python

"""Construct a GNU ld or Debian dpkg version-script from a set of
RFC822-style symbol lists.

Usage:
    make-version-script.py [--symbols SYMBOLS] [--unreleased-version VER]
        [--dpkg "LIBRARY.so.0 LIBRARY0 #MINVER#"]
        [--dpkg-build-depends-package LIBRARY-dev]
        [FILES...]

Each FILE starts with RFC822-style headers "Version:" (the name of the
symbol version, e.g. FOO_1.2.3) and "Extends:" (either the previous
version, or "-" if this is the first version). Next there is a blank
line, then a list of C symbols one per line.

Comments (lines starting with whitespace + "#") are allowed and ignored.

If --symbols is given, SYMBOLS lists the symbols actually exported by
the library (one per line). If --unreleased-version is given, any symbols
in SYMBOLS but not in FILES are assigned to that version; otherwise, any
such symbols cause an error.

If --dpkg is given, produce a Debian dpkg-gensymbols file instead of a
GNU ld version-script. The argument to --dpkg is the first line of the
resulting symbols file, and --dpkg-build-depends-package can optionally
be used to set the Build-Depends-Package field.

This script originates in telepathy-glib <http://telepathy.freedesktop.org/> -
please send us any changes that are needed.
"""

# Copyright (C) 2008 Collabora Ltd. <http://www.collabora.co.uk/>
# Copyright (C) 2008 Nokia Corporation
#
# Copying and distribution of this file, with or without modification,
# are permitted in any medium without royalty provided the copyright
# notice and this notice are preserved.

import sys
from getopt import gnu_getopt
from sets import Set as set


def e(format, *args):
    sys.stderr.write((format + '\n') % args)


def main(abifiles, symbols=None, unreleased_version=None,
         dpkg=False, dpkg_first_line=None, dpkg_build_depends_package=None):

    gnuld = not dpkg
    symbol_set = None

    if symbols is not None:
        symbol_set = open(symbols, 'r').readlines()
        symbol_set = map(str.strip, symbol_set)
        symbol_set = set(symbol_set)

    versioned_symbols = set()

    dpkg_symbols = []
    dpkg_versions = []

    if dpkg:
        assert dpkg_first_line is not None
        print dpkg_first_line
        if dpkg_build_depends_package is not None:
            print "* Build-Depends-Package: %s" % dpkg_build_depends_package

    for filename in abifiles:
        lines = open(filename, 'r').readlines()

        version = None
        extends = None
        release = None

        for i, line in enumerate(lines):
            line = line.strip()

            if line.startswith('#'):
                continue
            elif not line:
                # the transition betwen headers and symbols
                cut = i + 1
                break
            elif line.lower().startswith('version:'):
                line = line[8:].strip()
                version = line
                continue
            elif line.lower().startswith('extends:'):
                line = line[8:].strip()
                extends = line
                continue
            elif line.lower().startswith('release:'):
                release = line[8:].strip()
                continue
            else:
                e('Could not understand line in %s header: %s', filename, line)
                raise SystemExit(1)

        else:
            e('No symbols in %s', filename)
            raise SystemExit(1)

        if version is None:
            e('No Versions: header in %s', filename)
            raise SystemExit(1)

        if extends is None:
            e('No Extends: header in %s', filename)
            raise SystemExit(1)

        if release is None and dpkg:
            e('No Release: header in %s', filename)
            raise SystemExit(1)

        if dpkg:
            dpkg_versions.append('%s@%s %s' % (version, version, release))

        lines = lines[cut:]

        if gnuld:
            print "%s {" % version
            print "    global:"

        for symbol in lines:
            symbol = symbol.strip()

            if symbol.startswith('#'):
                continue

            if gnuld:
                print "        %s;" % symbol
            elif dpkg:
                dpkg_symbols.append('%s@%s %s' % (symbol, version, release))

            versioned_symbols.add(symbol)

        if gnuld:
            if extends == '-':
                print "    local:"
                print "        *;"
                print "};"
            else:
                print "} %s;" % extends
                print

    if dpkg:
        dpkg_symbols.sort()
        dpkg_versions.sort()

        for x in dpkg_versions:
            print " %s" % x

        for x in dpkg_symbols:
            print " %s" % x

    if symbol_set is not None:
        missing = versioned_symbols - symbol_set

        if missing:
            e('These symbols have disappeared:')

            for symbol in missing:
                e('    %s', symbol)

            raise SystemExit(1)

        unreleased = symbol_set - versioned_symbols

        if unreleased:
            if unreleased_version is None:
                e('Unversioned symbols are not allowed in releases:')

                for symbol in unreleased:
                    e('    %s', symbol)

                raise SystemExit(1)

            if gnuld:
                print "%s {" % unreleased_version
                print "    global:"

                for symbol in unreleased:
                    print "        %s;" % symbol

                print "} %s;" % version


if __name__ == '__main__':
    options, argv = gnu_getopt (sys.argv[1:], '',
                                ['symbols=', 'unreleased-version=',
                                 'dpkg=', 'dpkg-build-depends-package='])

    opts = {'dpkg': False}

    for option, value in options:
        if option == '--dpkg':
            opts['dpkg'] = True
            opts['dpkg_first_line'] = value
        else:
            opts[option.lstrip('-').replace('-', '_')] = value

    main(argv, **opts)



#!/usr/bin/python

import sys
import xml.dom.minidom

from libglibcodegen import NS_TP, get_docstring, get_descendant_text

class Generator(object):
    def __init__(self, dom):
        self.dom = dom
        self.errors = self.dom.getElementsByTagNameNS(NS_TP, 'errors')[0]

    def __call__(self):

        print '{'
        print '  GEnumClass *klass;'
        print '  GEnumValue *value_by_name;'
        print '  GEnumValue *value_by_nick;'
        print ''
        print '  g_type_init ();'
        print '  klass = g_type_class_ref (TP_TYPE_ERROR);'

        for error in self.errors.getElementsByTagNameNS(NS_TP, 'error'):
            ns = error.parentNode.getAttribute('namespace')
            nick = error.getAttribute('name').replace(' ', '')
            enum = ('TP_ERROR_' +
                    error.getAttribute('name').replace(' ', '_').replace('.', '_').upper())
            s = ('TP_ERROR_STR_' +
                 error.getAttribute('name').replace(' ', '_').replace('.', '_').upper())

            print ''
            print '  /* %s.%s */' % (ns, nick)
            print ('  value_by_name = g_enum_get_value_by_name (klass, "%s");'
                    % enum)
            print ('  value_by_nick = g_enum_get_value_by_nick (klass, "%s");'
                    % nick)
            print ('  g_assert (value_by_name != NULL);')
            print ('  g_assert (value_by_nick != NULL);')
            print ('  g_assert_cmpint (value_by_name->value, ==, %s);'
                    % enum)
            print ('  g_assert_cmpint (value_by_nick->value, ==, %s);'
                    % enum)
            print ('  g_assert_cmpstr (value_by_name->value_name, ==, "%s");'
                    % enum)
            print ('  g_assert_cmpstr (value_by_nick->value_name, ==, "%s");'
                    % enum)
            print ('  g_assert_cmpstr (value_by_name->value_nick, ==, "%s");'
                    % nick)
            print ('  g_assert_cmpstr (value_by_nick->value_nick, ==, "%s");'
                    % nick)
            print ('  g_assert_cmpstr (%s, ==, TP_ERROR_PREFIX ".%s");'
                    % (s, nick))

        print '}'

if __name__ == '__main__':
    argv = sys.argv[1:]
    Generator(xml.dom.minidom.parse(argv[0]))()



#!/usr/bin/python

import sys
import xml.dom.minidom
from string import ascii_letters, digits


from libglibcodegen import signal_to_marshal_name, method_to_glue_marshal_name


class Generator(object):

    def __init__(self, dom):
        self.dom = dom
        self.marshallers = {}

    def do_method(self, method):
        marshaller = method_to_glue_marshal_name(method, 'PREFIX')

        assert '__' in marshaller
        rhs = marshaller.split('__', 1)[1].split('_')

        self.marshallers[marshaller] = rhs

    def do_signal(self, signal):
        marshaller = signal_to_marshal_name(signal, 'PREFIX')

        assert '__' in marshaller
        rhs = marshaller.split('__', 1)[1].split('_')

        self.marshallers[marshaller] = rhs

    def __call__(self):
        methods = self.dom.getElementsByTagName('method')

        for method in methods:
            self.do_method(method)

        signals = self.dom.getElementsByTagName('signal')

        for signal in signals:
            self.do_signal(signal)

        all = self.marshallers.keys()
        all.sort()
        for marshaller in all:
            rhs = self.marshallers[marshaller]
            if not marshaller.startswith('g_cclosure'):
                print 'VOID:' + ','.join(rhs)

if __name__ == '__main__':
    argv = sys.argv[1:]
    dom = xml.dom.minidom.parse(argv[0])

    Generator(dom)()



"""Library code for GLib/D-Bus-related code generation.

The master copy of this library is in the telepathy-glib repository -
please make any changes there.
"""

# Copyright (C) 2006-2008 Collabora Limited
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


from libtpcodegen import NS_TP, \
                         Signature, \
                         cmp_by_name, \
                         escape_as_identifier, \
                         get_by_path, \
                         get_descendant_text, \
                         get_docstring, \
                         xml_escape, \
                         get_deprecated

def dbus_gutils_wincaps_to_uscore(s):
    """Bug-for-bug compatible Python port of _dbus_gutils_wincaps_to_uscore
    which gets sequences of capital letters wrong in the same way.
    (e.g. in Telepathy, SendDTMF -> send_dt_mf)
    """
    ret = ''
    for c in s:
        if c >= 'A' and c <= 'Z':
            length = len(ret)
            if length > 0 and (length < 2 or ret[length-2] != '_'):
                ret += '_'
            ret += c.lower()
        else:
            ret += c
    return ret


def signal_to_marshal_type(signal):
    """
    return a list of strings indicating the marshalling type for this signal.
    """

    mtype=[]
    for i in signal.getElementsByTagName("arg"):
        name =i.getAttribute("name")
        type = i.getAttribute("type")
        mtype.append(type_to_gtype(type)[2])

    return mtype


_glib_marshallers = ['VOID', 'BOOLEAN', 'CHAR', 'UCHAR', 'INT',
        'STRING', 'UINT', 'LONG', 'ULONG', 'ENUM', 'FLAGS', 'FLOAT',
        'DOUBLE', 'STRING', 'PARAM', 'BOXED', 'POINTER', 'OBJECT',
        'UINT_POINTER']


def signal_to_marshal_name(signal, prefix):

    mtype = signal_to_marshal_type(signal)
    if len(mtype):
        name = '_'.join(mtype)
    else:
        name = 'VOID'

    if name in _glib_marshallers:
        return 'g_cclosure_marshal_VOID__' + name
    else:
        return prefix + '_marshal_VOID__' + name


def method_to_glue_marshal_name(method, prefix):

    mtype = []
    for i in method.getElementsByTagName("arg"):
        if i.getAttribute("direction") != "out":
            type = i.getAttribute("type")
            mtype.append(type_to_gtype(type)[2])

    mtype.append('POINTER')

    name = '_'.join(mtype)

    if name in _glib_marshallers:
        return 'g_cclosure_marshal_VOID__' + name
    else:
        return prefix + '_marshal_VOID__' + name


def type_to_gtype(s):
    if s == 'y': #byte
        return ("guchar ", "G_TYPE_UCHAR","UCHAR", False)
    elif s == 'b': #boolean
        return ("gboolean ", "G_TYPE_BOOLEAN","BOOLEAN", False)
    elif s == 'n': #int16
        return ("gint ", "G_TYPE_INT","INT", False)
    elif s == 'q': #uint16
        return ("guint ", "G_TYPE_UINT","UINT", False)
    elif s == 'i': #int32
        return ("gint ", "G_TYPE_INT","INT", False)
    elif s == 'u': #uint32
        return ("guint ", "G_TYPE_UINT","UINT", False)
    elif s == 'x': #int64
        return ("gint64 ", "G_TYPE_INT64","INT64", False)
    elif s == 't': #uint64
        return ("guint64 ", "G_TYPE_UINT64","UINT64", False)
    elif s == 'd': #double
        return ("gdouble ", "G_TYPE_DOUBLE","DOUBLE", False)
    elif s == 's': #string
        return ("gchar *", "G_TYPE_STRING", "STRING", True)
    elif s == 'g': #signature - FIXME
        return ("gchar *", "DBUS_TYPE_G_SIGNATURE", "STRING", True)
    elif s == 'o': #object path
        return ("gchar *", "DBUS_TYPE_G_OBJECT_PATH", "BOXED", True)
    elif s == 'v':  #variant
        return ("GValue *", "G_TYPE_VALUE", "BOXED", True)
    elif s == 'as':  #array of strings
        return ("gchar **", "G_TYPE_STRV", "BOXED", True)
    elif s == 'ay': #byte array
        return ("GArray *",
            "dbus_g_type_get_collection (\"GArray\", G_TYPE_UCHAR)", "BOXED",
            True)
    elif s == 'au': #uint array
        return ("GArray *", "DBUS_TYPE_G_UINT_ARRAY", "BOXED", True)
    elif s == 'ai': #int array
        return ("GArray *", "DBUS_TYPE_G_INT_ARRAY", "BOXED", True)
    elif s == 'ax': #int64 array
        return ("GArray *", "DBUS_TYPE_G_INT64_ARRAY", "BOXED", True)
    elif s == 'at': #uint64 array
        return ("GArray *", "DBUS_TYPE_G_UINT64_ARRAY", "BOXED", True)
    elif s == 'ad': #double array
        return ("GArray *", "DBUS_TYPE_G_DOUBLE_ARRAY", "BOXED", True)
    elif s == 'ab': #boolean array
        return ("GArray *", "DBUS_TYPE_G_BOOLEAN_ARRAY", "BOXED", True)
    elif s == 'ao': #object path array
        return ("GPtrArray *",
                'dbus_g_type_get_collection ("GPtrArray",'
                ' DBUS_TYPE_G_OBJECT_PATH)',
                "BOXED", True)
    elif s == 'a{ss}': #hash table of string to string
        return ("GHashTable *", "DBUS_TYPE_G_STRING_STRING_HASHTABLE", "BOXED", False)
    elif s[:2] == 'a{':  #some arbitrary hash tables
        if s[2] not in ('y', 'b', 'n', 'q', 'i', 'u', 's', 'o', 'g'):
            raise Exception, "can't index a hashtable off non-basic type " + s
        first = type_to_gtype(s[2])
        second = type_to_gtype(s[3:-1])
        return ("GHashTable *", "(dbus_g_type_get_map (\"GHashTable\", " + first[1] + ", " + second[1] + "))", "BOXED", False)
    elif s[:2] in ('a(', 'aa'): # array of structs or arrays, recurse
        gtype = type_to_gtype(s[1:])[1]
        return ("GPtrArray *", "(dbus_g_type_get_collection (\"GPtrArray\", "+gtype+"))", "BOXED", True)
    elif s[:1] == '(': #struct
        gtype = "(dbus_g_type_get_struct (\"GValueArray\", "
        for subsig in Signature(s[1:-1]):
            gtype = gtype + type_to_gtype(subsig)[1] + ", "
        gtype = gtype + "G_TYPE_INVALID))"
        return ("GValueArray *", gtype, "BOXED", True)

    # we just don't know ..
    raise Exception, "don't know the GType for " + s
